@@ -1,10 +1,11 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 
 """
-Copyright (c) 2006-2019 sqlmap developers (http://sqlmap.org/)
+Copyright (c) 2006-2020 sqlmap developers (http://sqlmap.org/)
 See the file 'LICENSE' for copying permission
 """
 
+from lib.core.convert import getText
 from thirdparty.six.moves import urllib as _urllib
 
 class MethodRequest(_urllib.request.Request):
@@ -13,7 +14,7 @@ class MethodRequest(_urllib.request.Request):
     """
 
     def set_method(self, method):
-        self.method = method.upper()
+        self.method = getText(method.upper())  # Dirty hack for Python3 (may it rot in hell!)
 
     def get_method(self):
         return getattr(self, 'method', _urllib.request.Request.get_method(self))
